/*
 * 3D Morph Toolkit
 *
 * (c) 2000 Sylvain Tertois
 *
 *
 * MTVector: float vector
 */

#ifndef MTVECTOR_H
#define MTVECTOR_H

class MTPoints;

class MTVector
{
public:
	MTVector();
	MTVector(unsigned int nValues, const float *values);
	MTVector(const MTVector &);
	~MTVector();
	
	// assignement functions
	MTVector &SetTo(const MTVector&);
	inline MTVector &operator=(const MTVector &src)
		{ return SetTo(src); }
	MTVector &SetTo(unsigned int nValues, const float *values);
	
	// operators
	// add some values to the list
	MTVector &AddValues(const MTVector&);
	inline MTVector &operator&=(const MTVector &src)
		{ return AddValues(src); }
	MTVector operator&(const MTVector &src) const;
	MTVector &AddValue(float);
	inline MTVector &operator&=(float src)
		{ return AddValue(src); }
	MTVector operator&(float) const;
	
	// repeat
	void Repeat(unsigned int each, unsigned int list);
	
	// mathematic operations between vectors
	// both vectors must have the same size
	MTVector &operator+=(const MTVector&);
	MTVector operator+(const MTVector&) const;
	MTVector &operator-=(const MTVector&);
	MTVector operator-(const MTVector&) const;
	MTVector &operator*=(const MTVector&);
	MTVector operator*(const MTVector&) const;
	MTVector &operator/=(const MTVector&);
	MTVector operator/(const MTVector&) const;

	// trigonometric functions (in radians)
	// these function are applied on the object
	void sin();
	void cos();
	void tan();

	// mathematic operations with a float
	MTVector &operator+=(float);
	MTVector operator+(float) const;
	MTVector &operator-=(float);
	MTVector operator-(float) const;
	MTVector &operator*=(float);
	MTVector operator*(float) const;
	MTVector &operator/=(float);
	MTVector operator/(float) const;

	// access to individual values (if index is out of range, the vector's 
	// last value is returned)
	float &operator[](unsigned int index) const;

	// access to the number of values, and the buffer
	inline unsigned int NumValues() const
		{ return m_NumValues; }
	inline const float *Values() const
		{ return m_Values; }
	
	// indirection operator
	MTVector operator^(const MTVector&) const;

	// basic vectors
	static MTVector Constant(float constant, unsigned int nValues);
	static MTVector GradientInc(float val1, float val2, unsigned int nValues);
	static MTVector GradientExc(float val1, float val2, unsigned int nValues);
	static MTVector Random(float val1,float val2,unsigned int nValues);

private:
	unsigned int m_NumValues;
	float *m_Values;
};

// other trigonometric operators
extern MTVector sin(const MTVector&);
extern MTVector cos(const MTVector&);
extern MTVector tan(const MTVector&);

#endif //MTVECTOR_H
